@@ -1,11 +1,13 @@
 #' SilhouetR
 #'
-#' Calculate the Silhouette value for 
+#' This function calculate Silhouette values. The function is generic, 
+#' as such silhouette values can be calculated between samples of different classes or 
+#' it can be used to calculate silhouette values between different groups of peaks. 
+#' This is the way in which it is used for the speaq package (see the example).
 #'
 #' @param DataMatrix a matrix with the raw data, 1 variable per column.
 #' @param GroupIndices The vector with the group indices (length must be equal to the amount of rows in DataMatrix).
-#' @param distance The distance metric to be used, see \link[cluster]{daisy}.
-#' @param stand whether to standardize the data before calculating the dissimilarities. See \link[cluster]{daisy}.
+#' @param distance The distance metric to be used, "euclidean" or "manhattan".
 #'
 #' @return Returns the silhouette values. Note if a group contains only 1 no Silhouette value can be calculated (will give NA)
 #' 
@@ -25,24 +27,29 @@
 #'
 #' Silhouette.values = SilhouetR(DataMatrix = test.grouped$peakPPM, 
 #'                               test.grouped$peakIndex, 
-#'                               distance = 'euclid', 
-#'                               stand = TRUE)
+#'                               distance = "euclidean")
 #'                               
 #' hist(Silhouette.values$SilhouetteValues)
 #' 
 #' 
 #' @export
 #' 
-#' @importFrom cluster daisy
+#' @importFrom Rfast dista
 #' @importFrom utils head tail txtProgressBar
 #' 
-SilhouetR <- function(DataMatrix, GroupIndices, distance = "euclid", stand = TRUE) {
+SilhouetR <- function(DataMatrix, GroupIndices, distance = "euclidean") {
     
     if (!"matrix" %in% class(DataMatrix) & !"data.frame" %in% class(DataMatrix)) {
         DataMatrix <- matrix(DataMatrix, ncol = 1)
         message("DataMatrix is not a matrix, attempting conversion with the assumption of only 1 variable (1 column)")
     }
     
+    if(any(is.na(GroupIndices))){
+        message("GroupIndices contains NA values. These will be deleted together with the corresponding DataMatrix entries.")
+        DataMatrix = DataMatrix[!is.na(GroupIndices), , drop = FALSE]
+        GroupIndices = GroupIndices[!is.na(GroupIndices)]
+    }
+    
     # order the Datamatrix according to increasing GroupIndex, this is to speed up the code later on by
     # calling indices instead of comparing to group
     DataMatrix <- DataMatrix[order(GroupIndices), , drop = FALSE]
@@ -54,18 +61,21 @@ SilhouetR <- function(DataMatrix, GroupIndices, distance = "euclid", stand = TRU
     Nindividuals <- nrow(DataMatrix)
     individual_indices <- 1:Nindividuals
     
-    start_stop_indices <- matrix(NA, ncol = 2, nrow = Ngroups)
-    for (k in 1:Ngroups) {
-        start_stop_indices[k, 1] <- utils::head(which(GroupIndices == groups[k]), n = 1)
-        start_stop_indices[k, 2] <- utils::tail(which(GroupIndices == groups[k]), n = 1)
+    
+    if(Ngroups > 2){
+        start_stop_indices <- matrix(NA, ncol = 2, nrow = Ngroups)
+        start_stop_indices[1,1] <- 1
+        start_stop_indices[Ngroups,2] <- length(GroupIndices)
+        start_stop_indices[1:(Ngroups-1),2] <- which(diff(GroupIndices)>0)
+        start_stop_indices[2:Ngroups,1] <- which(diff(GroupIndices)>0) + 1
+    } else{
+        start_stop_indices <- matrix(NA, ncol = 2, nrow = Ngroups)
+        for (k in 1:Ngroups) {
+            start_stop_indices[k, 1] <- utils::head(which(GroupIndices == groups[k]), n = 1)
+            start_stop_indices[k, 2] <- utils::tail(which(GroupIndices == groups[k]), n = 1)
+        }
     }
     
-    
-    
-    print("Computing dissimilarity matrix")
-    # dissimilarity matrix
-    dissSim <- as.matrix(cluster::daisy(DataMatrix, metric = distance, stand = stand))
-    
     print("Computing silhouette values")
     pb <- utils::txtProgressBar(min = 0, max = Nindividuals, style = 3, width = 100)
     
@@ -74,14 +84,19 @@ SilhouetR <- function(DataMatrix, GroupIndices, distance = "euclid", stand = TRU
     SilhouetteValues <- rep(NA, Nindividuals)
     for (i in 1:Nindividuals) {
         ABvalues <- rep(NA, Ngroups)
+        if(ncol(DataMatrix) == 1){
+            dissSim <- abs(DataMatrix - DataMatrix[i,])
+        } else{
+            dissSim <- Rfast::dista(xnew = DataMatrix[i, , drop = FALSE], x = DataMatrix, type = distance )
+        }
         for (grp in 1:Ngroups) {
-            # ABvalues[grp] = mean(dissSim[i,GroupIndices == groups[grp] & individual_indices!=i])
-            ABvalues[grp] <- mean(dissSim[i, start_stop_indices[grp, 1]:start_stop_indices[grp, 2]])
+            ABvals <- dissSim[start_stop_indices[grp, 1]:start_stop_indices[grp, 2]]
+            ABvalues[grp] <- sum(ABvals)/length(ABvals)
         }
         individual_grp <- which(groups == GroupIndices[i])
         # A = ABmatrix[i,individual_grp] B = min(ABmatrix[i,-individual_grp])
-        ABvalues[individual_grp] <- mean(dissSim[i, GroupIndices == GroupIndices[i] & individual_indices != 
-            i])
+        ABvalues[individual_grp] <- mean(dissSim[GroupIndices == GroupIndices[i] & individual_indices != 
+                                                     i])
         A <- ABvalues[individual_grp]
         B <- min(ABvalues[-individual_grp])
         SilhouetteValues[i] <- (B - A)/max(c(A, B))
@@ -89,7 +104,7 @@ SilhouetR <- function(DataMatrix, GroupIndices, distance = "euclid", stand = TRU
     }
     close(pb)
     SilhouetteValues.df <- data.frame(matrix(c(original.order[individual_indices], SilhouetteValues, GroupIndices), ncol = 3, 
-        byrow = FALSE))
+                                             byrow = FALSE))
     colnames(SilhouetteValues.df) <- c("index", "SilhouetteValues", "GroupIndices")
     
     SilhouetteValues.df$SilhouetteValues[is.nan(SilhouetteValues.df$SilhouetteValues)] = NA
@@ -97,4 +112,5 @@ SilhouetR <- function(DataMatrix, GroupIndices, distance = "euclid", stand = TRU
     SilhouetteValues.df = SilhouetteValues.df[order(original.order),]
     
     return(SilhouetteValues.df)
-}
+    
+}
\ No newline at end of file
